package edu.unl.cse.csce361.hibernate;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.boot.MetadataSources;
import org.hibernate.boot.registry.StandardServiceRegistryBuilder;
import org.hibernate.service.ServiceRegistry;

public class FruitPopulator {
    private static Session session = null;

    public static Session getSession() {
        if (session == null) {
            SessionFactory sessionFactory = null;
            ServiceRegistry serviceRegistry = new StandardServiceRegistryBuilder().configure().build();
            try {
                sessionFactory = new MetadataSources(serviceRegistry).buildMetadata().buildSessionFactory();
            } catch (Exception exception) {
                // sessionFactory didn't have the opportunity to destroy serviceRegistry like it should've
                StandardServiceRegistryBuilder.destroy(serviceRegistry);
                System.err.println("Failed to create session.  " + exception);
                System.exit(1);
            }
            session = sessionFactory.openSession();
        }
        return session;
    }

    public static void closeSession(Session session) {
        if (session != null) {
            try {
                session.close();
            } catch (HibernateException hibernateException) {
                System.err.println("Failed to close session due to Hibernate problem.  " + hibernateException);
                System.err.println("Abandoning possibly-open session.");
            } finally {
                //noinspection UnusedAssignment
                session = null;
            }
        }
    }

    private void makeFruitSalad() {
        Session session = getSession();
        try {
            session.beginTransaction();
            Fruit apple = new Fruit("apple", "red", 80);
            Fruit banana = new Fruit("banana", "yellow", 27);
            Fruit orange = new Fruit("orange", "orange");
            session.save(apple);
            session.save(banana);
            session.save(orange);
            FruitSalad fruitSalad = new FruitSalad();
            fruitSalad.setRecipeName("Simple Fruit Salad");
            session.save(fruitSalad);
            fruitSalad.addFruit(apple);
            fruitSalad.addFruit(banana);
            session.getTransaction().commit();
            System.out.println();
            System.out.println("Successfully populated tables.");
        } catch (RuntimeException exception) {
            session.getTransaction().rollback();
            System.out.println();
            System.out.println("Failed to populate tables.  " + exception);
        }
    }

    private void examineFruitSalad() {
        Session session = getSession();
        session.beginTransaction();
        FruitSalad fruitSalad = session.load(FruitSalad.class, 4);  // This is INCREDIBLY fragile
        session.getTransaction().commit();
        System.out.print(fruitSalad + " contains ");
        for (Fruit fruit : fruitSalad.getFruits()) {
            System.out.print(fruit + " and ");
        }
        System.out.println("that is all.");
    }

    public static void main(String[] args) {
        FruitPopulator fruitPopulator = new FruitPopulator();
        fruitPopulator.makeFruitSalad();
        fruitPopulator.examineFruitSalad();
        closeSession(session);
    }
}
